; Modified blinking t-shirt code.
; This code has been modified somewhat since the original article was written.
; It has had the labels revised to fit a newer shirt with 3 parrots
; and a shark on it.  It will work correctly, for the guitar t-shirt described
; in the article as long as you connect the guitar strings to pins 1,2,4 and 5
; on the PIC processor as described in the article. 
#include <p12F675.inc>
	__config (_INTRC_OSC_NOCLKOUT & _WDT_OFF & _PWRTE_OFF & _MCLRE_OFF & _CP_OFF )

	cblock 0x20
Delay1			; Define two file registers for the
Delay2			; delay loop
Display 		; define a variable to hold the diplay
Direction 
SwitchUp
T0Semaphore
BigLoopCount
Counter
W_Save3			; for fast rotate
W_Save2
W_Save1
STATUS_Save
SharkOn
SaveDir
	endc

GrnBrd	equ	2	
YlwBrd	equ	1
RedBrd	equ 5
BluShrk	equ 4
SharkDelay	equ	4
	org 0
	goto	Start
	nop
	nop
	nop

DelayLoop
;delay as long as amount in Delay1,Delay2 (delay2 is MSB)
	movf	BigLoopCount,0	; get 8 MSB in f
	movwf	Delay2		; store 8 MSB in our delay counter
	movf	W_Save1,0
	movwf	Delay1
; 10/10/06- Add code to test the switch status in the delay loop
DelayLoop1

; Here's how I think this code works.  First, I check to see if the switch is pressed.
; If it IS, I set the flag in Counter that it HAS been pressed and continue the loop.
; If is not currently being pressed, then I check the Counter flag to see if it HAS
; BEEN pressed.  If it has, then I reset the counter flag and increment Direction.
; If direction overflows, I reset it to 1.   
;  
	btfsc	GPIO,3			; is the switch pressed (0)
	Goto	DelayLoop2
	bsf		Counter,0		;yes, set flag
	Goto	DelayLoop3
DelayLoop2					;If here, switch is UP
	btfss	Counter,0		;has counter been set (i.e. switch was down)?
	Goto	DelayLoop3		;it WAS NOT set, skip over display change code	
	Movlw	0				;reset counter
	Movwf	Counter
	Rlf		Direction,1		; Yes, it was pushed-rotate active bit
	btfss	03,0			; carry flag set?
	Return					; it's clear, just rotate & exit
	Movlw	1				; no, it's set
	Movwf	Direction		; set first bit	
	
DelayLoop3	
	decfsz	Delay1,f		; Waste time.  
	goto	DelayLoop1		; The Inner loop takes 6 instructions per loop * 256 loops = 1536 instructions
	decfsz	Delay2,f		; The outer loop takes an additional 3 instructions per lap * 256 loops
	goto	DelayLoop1		; (768+3) * 256 = 197376 instructions / 1M instructions per second = 0.197 sec.
	return					; call it a two-tenths of a second.

FastLoop
	Movf	W_Save3,0		;fast loop counter
	Movwf	BigLoopCount
	Call	BirdLoop
	Movf	W_Save2,0
	Movwf	BigLoopCount
	Return
	
BlinkYelBrd					;Yellow bird
	bsf		Display,YlwBrd		;turn on bit 1
	Movf	Display,0		;move DISPLAY into W
	movwf	GPIO			;store
	Call	DelayLoop
	bcf		Display,YlwBrd		;turn off bit 1
	Movf	Display,0		;move DISPLAY into W
	movwf	GPIO			;store

	Return
BlinkGrnBrd					;Green Bird
	bsf		Display,GrnBrd		;turn on bit 2
	Movf	Display,0		;move DISPLAY into W
	movwf	GPIO			;store
	
	Call	DelayLoop
	bcf		Display,GrnBrd		;turn off bit 2
	Movf	Display,0		;move DISPLAY into W
	movwf	GPIO			;store

	Return

BlinkBluShrk				;Shark
	bsf		Display,BluShrk		;turn on bit 4
	Movf	Display,0		;move DISPLAY into W
	movwf	GPIO			;store
	Call	DelayLoop
	bcf		Display,BluShrk		;turn off bit 4
	Movf	Display,0		;move DISPLAY into W
	movwf	GPIO			;store
;	Call	DelayLoop
	Return

BlinkRedBrd					;Red Bird
	bsf		Display,RedBrd		;turn on bit 0
	Movf	Display,0		;move DISPLAY into W
	movwf	GPIO			;store
	Call	DelayLoop
	bcf		Display,RedBrd		;turn on bit 1
	Movf	Display,0		;move DISPLAY into W
	movwf	GPIO			;store

	Return

AllBirdsOn
	Movf	Display,0		;move DISPLAY into W
	iorlw	b'110111'
	movwf	Display			;put it back!
	movwf	GPIO			;and output it
	Return
AllBirdsOff
	Movf	Display,0		;move DUSPLAY into W
	andlw	b'001000'		;MSB first??
	movwf	Display
	movwf	GPIO
	Return
BirdLoop
;This blinks the Strings in sequence set by the Direction flag

BirdLoop2
	btfss	Direction,0		;Is bit 0 set?
	goto	OtherDir		;It's clear
	Call	BlinkRedBrd
	Call	BlinkBluShrk
    Call	BlinkYelBrd
	Call	BlinkGrnBrd
	Call	DelayLoop
	Return
OtherDir
    Call	BlinkGrnBrd
	Call	BlinkYelBrd
	Call	BlinkBluShrk
	Call	BlinkRedBrd
	Call	DelayLoop
	Return
BlinkOne
	Call	AllBirdsOn
	Btfsc	Direction,4
	Call	BlinkYelBrd			;Yellow bird- one stays on, two blink
	Btfsc	Direction,5
	Call	BlinkGrnBrd			;Green bird
	Call	BlinkBluShrk
	Btfsc	Direction,6
	Call	BlinkRedBrd			;Red bird
	Return

StayOn
	Call	AllBirdsOn
	Call	DelayLoop			;This will this leave all the birds on
; there must be a delay here, or the switch doesn't get checked.
    btfsc	Direction,0			;has button been pressed?, when bit 1 goes high, exit
	goto	StayOn
	Return

Start
	bsf		STATUS,RP0	; select Register Page 1
    banksel TRISIO
	bcf 	TRISIO,1	; pin 1 is output
	bcf		TRISIO,2	; Pin 2
	bcf		TRISIO,4	; Pin 4
	bcf		TRISIO,5	; pin 5
	bcf		TRISIO,0	; Pin 7, now output (currently unused)
	bsf		TRISIO,3	; switch, input
	banksel	GPIO
;	bcf		STATUS,RP0	; back to Register Page 0
    clrf 	GPIO
	clrf	SwitchUp	; Looking for a 0 on the button
	Movlw	1		
	Movwf	Direction	;switch code uses this
	Movlw	0
	Movwf	Counter		;for switch detect routine
	Movwf	Display		;start with all off
;init delay variables
    movlw	0xF0
	movwf	W_Save1
	movlw	0x80		;try faster loop
	movwf	W_Save2
	Movwf	BigLoopCount
	Movlw	0x20
	Movwf	W_Save3		;for very fast loop
	movlw	0
	movwf	Delay2
	Movlw	SharkDelay
    Movwf	SharkOn
;Testing code to make sure that all circuits work
;	Call	AllBirdsOn
;	Call	DelayLoop
;	Call	AllBirdsOff
;	Call	DelayLoop
;	Goto	Testing

	bsf		Display,4
	Call	StayOn
;This doesn't return until the button is pressed
	Movlw	1			;Now reset direction flag		
	Movwf	Direction	;switch code uses this
	
MainLoop
	
	Btfsc	Direction,0			;rotate one way
	Call	BirdLoop
	Btfsc	Direction,1			;rotate other way
	Call	BirdLoop 
	Btfsc	Direction,2			;All Birds on
	Call	FastLoop
	Btfsc	Direction,3
	Call	AllBirdsOn
;	Call	BlinkBluShrk		;the shark blinks with birds on
	Btfsc	Direction,4
	Call	BlinkOne			;Yellow bird- one stays on, two blink
	Btfsc	Direction,5
	Call	BlinkOne			;Green bird
	Btfsc	Direction,6
	Call	BlinkOne			;Red bird
	Btfsc	Direction,7
	Call	DelayLoop			;Give it a chance to test the switch
	Goto	MainLoop			


	end
	
